@@ -107,7 +107,7 @@ formatting = {
             'detailedWhereAmI' : '[]'
             },
         pyatspi.ROLE_ALERT: {
-            'unfocused': 'labelOrName + unrelatedLabels'
+            'unfocused': 'expandedEOCs or (labelOrName + unrelatedLabels)'
             },
         pyatspi.ROLE_ANIMATION: {
             'unfocused': 'labelAndName'
@@ -145,7 +145,7 @@ formatting = {
             },
         pyatspi.ROLE_DIALOG: {
             'focused': 'labelOrName',
-            'unfocused': 'labelOrName + unrelatedLabels'
+            'unfocused': 'expandedEOCs or (labelOrName + unrelatedLabels)'
             },
         pyatspi.ROLE_DOCUMENT_FRAME: {
             'unfocused': 'label + readOnly + textRole + textContent + anyTextSelection + ' + MNEMONIC,
@@ -220,7 +220,7 @@ formatting = {
         pyatspi.ROLE_MENU_ITEM: {
             'focused': 'expandableState',
             'unfocused': 'labelOrName + menuItemCheckedState + expandableState + availability + ' + MNEMONIC + ' + accelerator + positionInList',
-            'basicWhereAmI': 'ancestors + labelOrName + accelerator + positionInList + ' + MNEMONIC
+            'basicWhereAmI': 'ancestors + parentRoleName + labelOrName + accelerator + positionInList + ' + MNEMONIC
             },
         pyatspi.ROLE_NOTIFICATION: {
             'unfocused': 'roleName + unrelatedLabels'
@@ -399,7 +399,10 @@ formatting = {
             'unfocused': '[Component(obj,\
                                      asString(label + displayedText + value + roleName + required))]',
             },
-        #pyatspi.ROLE_ALERT: 'default'
+        pyatspi.ROLE_ALERT: {
+            'unfocused': '((substring and ' + BRAILLE_TEXT + ')\
+                          or ([Component(obj, asString(labelAndName + roleName))]))'
+            },
         pyatspi.ROLE_ANIMATION: {
             'unfocused': '[Component(obj,\
                                      asString(label + displayedText + roleName + (description and space(": ") + description)))]',
@@ -425,9 +428,9 @@ formatting = {
             # We could then use the cursorOffset field to indicate where the
             # combobox starts.]]]
             #
-            'unfocused': '((comboBoxTextObj and [Text(comboBoxTextObj[0], asString(label), asString(eol))])\
-                           or [Component(obj, asString(label + name), label and (len(asString(label)) + 1) or 0)])\
-                          + [Region(" " + asString(roleName))]'
+            'unfocused': '((comboBoxTextObj and ([Text(comboBoxTextObj[0], asString(label), asString(eol))] \
+                                               + [Region(" " + asString(roleName))])) \
+                           or [Component(obj, asString(label + name + roleName), label and (len(asString(label)) + 1) or 0)])'
             },
         #pyatspi.ROLE_DESKTOP_ICON: 'default'
         pyatspi.ROLE_DIAL: {
@@ -451,6 +454,10 @@ formatting = {
         pyatspi.ROLE_ENTRY: {
             'unfocused': BRAILLE_TEXT
             },
+        pyatspi.ROLE_FORM: {
+            'unfocused': '((substring and ' + BRAILLE_TEXT + ')\
+                          or ([Component(obj, asString(labelAndName + roleName))]))'
+            },
         pyatspi.ROLE_FRAME: {
             'focused':   '[Component(obj,\
                                      asString(((label + displayedText) or name) + value + roleName + alertAndDialogCount))]',
@@ -458,7 +465,8 @@ formatting = {
                                      asString(((label + displayedText) or name) + value + roleName + alertAndDialogCount))]'
             },
         pyatspi.ROLE_HEADING: {
-            'unfocused': '[Text(obj)] + [Region(" " + asString(roleName))]'
+            'unfocused': '[Text(obj, asString(placeholderText), "", startOffset, endOffset)]\
+                          + [Region(" " + asString(roleName))]'
             },
         #pyatspi.ROLE_HTML_CONTAINER: 'default'
         pyatspi.ROLE_ICON: {
@@ -477,7 +485,8 @@ formatting = {
         pyatspi.ROLE_LINK: {
             'unfocused': '[Link(obj, asString(currentLineText)\
                                      or asString(displayedText)\
-                                     or asString(name))]',
+                                     or asString(name))] \
+                        + (roleName and [Region(" " + asString(roleName))])',
         },
         pyatspi.ROLE_LIST: {
             'unfocused': '[Component(obj,\
