@@ -29,9 +29,17 @@ __license__   = "LGPL"
 
 import pyatspi
 
+import orca.orca as orca
 import orca.scripts.toolkits.gtk as gtk
 import orca.scripts.toolkits.WebKitGtk as WebKitGtk
 import orca.settings as settings
+import orca.settings_manager as settings_manager
+
+from .braille_generator import BrailleGenerator
+from .speech_generator import SpeechGenerator
+from .script_utilities import Utilities
+
+_settingsManager = settings_manager.getManager()
 
 ########################################################################
 #                                                                      #
@@ -48,9 +56,21 @@ class Script(WebKitGtk.Script):
         - app: the application to create a script for.
         """
 
-        WebKitGtk.Script.__init__(self, app, False)
+        if _settingsManager.getSetting('sayAllOnLoad') == None:
+            _settingsManager.setSetting('sayAllOnLoad', False)
+
+        super().__init__(app)
         self.presentIfInactive = False
 
+    def getBrailleGenerator(self):
+        return BrailleGenerator(self)
+
+    def getSpeechGenerator(self):
+        return SpeechGenerator(self)
+
+    def getUtilities(self):
+        return Utilities(self)
+
     def isActivatableEvent(self, event):
         """Returns True if the given event is one that should cause this
         script to become the active script.  This is only a hint to
@@ -95,6 +115,13 @@ class Script(WebKitGtk.Script):
         if self.utilities.isWebKitGtk(event.source):
             return
 
+        # This is some mystery child of the 'Messages' panel which fails to show
+        # up in the hierarchy or emit object:state-changed:focused events.
+        if event.source.getRole() == pyatspi.ROLE_LAYERED_PANE:
+            obj = self.utilities.realActiveDescendant(event.source)
+            orca.setLocusOfFocus(event, obj)
+            return
+
         gtk.Script.onFocus(self, event)
 
     def onNameChanged(self, event):
