@@ -37,6 +37,7 @@ from . import settings
 from . import settings_manager
 from . import sound
 from . import text_attribute_names
+from . import acss
 
 class Pause:
     """A dummy class to indicate we want to insert a pause into an
@@ -223,7 +224,9 @@ class SpeechGenerator(generator.Generator):
         return result
 
     def _generateClickable(self, obj, **args):
-        if _settingsManager.getSetting('onlySpeakDisplayedText'):
+        if _settingsManager.getSetting('onlySpeakDisplayedText') \
+           or _settingsManager.getSetting('speechVerbosityLevel') \
+               == settings.VERBOSITY_LEVEL_BRIEF:
             return []
 
         acss = self.voice(SYSTEM)
@@ -242,6 +245,26 @@ class SpeechGenerator(generator.Generator):
             result.extend(acss)
         return result
 
+    def _generateAvailability(self, obj, **args):
+        if _settingsManager.getSetting('onlySpeakDisplayedText'):
+            return []
+
+        acss = self.voice(SYSTEM)
+        result = generator.Generator._generateAvailability(self, obj, **args)
+        if result:
+            result.extend(acss)
+        return result
+
+    def _generateRequired(self, obj, **args):
+        if _settingsManager.getSetting('onlySpeakDisplayedText'):
+            return []
+
+        acss = self.voice(SYSTEM)
+        result = generator.Generator._generateRequired(self, obj, **args)
+        if result:
+            result.extend(acss)
+        return result
+
     def _generateTextRole(self, obj, **args):
         """A convenience method to prevent the pyatspi.ROLE_PARAGRAPH role
         from being spoken. In the case of a pyatspi.ROLE_PARAGRAPH
@@ -281,8 +304,16 @@ class SpeechGenerator(generator.Generator):
                         pyatspi.ROLE_FILLER,
                         pyatspi.ROLE_EXTENDED]
 
+        try:
+            parentRole = obj.parent.getRole()
+        except:
+            parentRole = None
+
+        if role == pyatspi.ROLE_MENU and parentRole == pyatspi.ROLE_COMBO_BOX:
+            return self._generateRoleName(obj.parent)
+
         # egg-list-box, e.g. privacy panel in gnome-control-center
-        if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_BOX:
+        if parentRole == pyatspi.ROLE_LIST_BOX:
             doNotPresent.append(obj.getRole())
 
         if _settingsManager.getSetting('speechVerbosityLevel') \
@@ -938,12 +969,20 @@ class SpeechGenerator(generator.Generator):
         """
         acss = self.voice(DEFAULT)
         result = generator.Generator._generateCurrentLineText(self, obj, **args)
-        if result and result[0]:
-            if result[0] == "\n":
-                result[0] = messages.BLANK
+        if result:
+            if result == ["\n"]:
+                result = [messages.BLANK]
             result.extend(acss)
         return result
 
+    def _generateDisplayedText(self, obj, **args):
+        acss = self.voice(DEFAULT)
+        result = generator.Generator._generateDisplayedText(self, obj, **args)
+        if result:
+            result.extend(acss)
+
+        return result
+
     def _getCharacterAttributes(self,
                                 obj,
                                 text,
@@ -1140,6 +1179,11 @@ class SpeechGenerator(generator.Generator):
         if _settingsManager.getSetting('onlySpeakDisplayedText'):
             return []
 
+        try:
+            text = obj.queryText()
+        except NotImplementedError:
+            return []
+
         result = []
         acss = self.voice(SYSTEM)
 
@@ -1563,7 +1607,7 @@ class SpeechGenerator(generator.Generator):
            and args.get('formatType', None) \
                in ['basicWhereAmI', 'detailedWhereAmI']:
             return [object_properties.ROLE_ICON_PANEL]
-        elif obj.parent.getRole() == pyatspi.ROLE_TABLE_CELL:
+        if obj.parent.getRole() in [pyatspi.ROLE_TABLE_CELL, pyatspi.ROLE_MENU]:
             obj = obj.parent
         return self._generateRoleName(obj.parent)
 
@@ -1624,6 +1668,7 @@ class SpeechGenerator(generator.Generator):
         specifications) that represent the relative position of an
         object in a list.
         """
+
         if _settingsManager.getSetting('onlySpeakDisplayedText') \
            or not (_settingsManager.getSetting('enablePositionSpeaking') \
                    or args.get('forceList', False)):
@@ -1631,80 +1676,17 @@ class SpeechGenerator(generator.Generator):
 
         result = []
         acss = self.voice(SYSTEM)
-        position = -1
-        index = 0
-        total = 0
-        name = self._generateName(obj)
-        # TODO - JD: There might be a better way to do this (e.g. pass
-        # roles in maybe?).
-        #
-        role = args.get('role', obj.getRole())
-        if role == pyatspi.ROLE_COMBO_BOX:
-            obj = obj[0]
-        elif role in [pyatspi.ROLE_PAGE_TAB,
-                      pyatspi.ROLE_MENU,
-                      pyatspi.ROLE_MENU_ITEM,
-                      pyatspi.ROLE_CHECK_MENU_ITEM,
-                      pyatspi.ROLE_RADIO_MENU_ITEM]:
-            obj = obj.parent
-        elif role == pyatspi.ROLE_LIST_ITEM:
-            parent = obj.parent
-            for relation in obj.getRelationSet():
-                if relation.getRelationType() == \
-                        pyatspi.RELATION_NODE_CHILD_OF:
-                    # childNodes assumes that we have an accessible table
-                    # interface to work with. If we don't, it will fail. So
-                    # don't set the parent until verifying the interface we
-                    # expect actually exists.
-                    #
-                    target = relation.getTarget(0)
-                    try:
-                        target.parent.queryTable()
-                    except:
-                        pass
-                    else:
-                        parent = target
-                    break
-            obj = parent
-        else:
-            obj = obj.parent
+        position, total = self._script.utilities.getPositionAndSetSize(obj)
+        if position < 0 or total < 0:
+            return []
 
-        # We want to return the position relative to this hierarchical
-        # level and not the entire list.  If the object in question
-        # uses the NODE_CHILD_OF relationship, we need to use it instead
-        # of the childCount.
-        #
-        childNodes = self._script.utilities.childNodes(obj)
-        total = len(childNodes)
-        for i in range(0, total):
-            childName = self._generateName(childNodes[i])
-            if childName == name:
-                position = i+1
-                break
-
-        if not total:
-            for child in obj:
-                nextName = self._generateName(child)
-                state = child.getState()
-                if not nextName or nextName[0] in ["", "Empty", "separator"] \
-                   or not state.contains(pyatspi.STATE_VISIBLE):
-                    continue
-
-                index += 1
-                total += 1
-
-                if nextName == name:
-                    position = index
-
-        if (_settingsManager.getSetting('enablePositionSpeaking') \
-             or args.get('forceList', False)) \
-           and position >= 0:
-            result.append(self._script.formatting.getString(
+        position += 1
+        result.append(self._script.formatting.getString(
                               mode='speech',
                               stringType='groupindex') \
                           % {"index" : position,
                              "total" : total})
-            result.extend(acss)
+        result.extend(acss)
         return result
 
     def _generateDefaultButton(self, obj, **args):
@@ -1878,7 +1860,7 @@ class SpeechGenerator(generator.Generator):
         voicename = voiceType.get(key) or voiceType.get(DEFAULT)
         voices = _settingsManager.getSetting('voices')
         rv = voices.get(voicename)
-        if rv and rv.get('established') == False:
-            rv.pop('established')
+        if not rv or rv.get('established') == False:
+            rv = voices.get(voiceType.get(DEFAULT))
 
-        return [rv]
+        return [acss.ACSS(rv)]
